/*
	Copyright Berend "Kirk" Wouda 2004

	This file is part of WhatDoYouSee'd.

	WhatDoYouSee'd is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	WhatDoYouSee'd is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with WhatDoYouSee'd; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// The package, d'oh.
package WDYSDecrypt;

// Import the IO classes.
import java.io.*;

// Import the helper class.
import system.HelperMethods;

/**
 * This class reads encrypted WDYS files, isolates the encrypted data chunk and
 * provides methods for data retrieval and stuff.
 *
 * @author Berend "Kirk" Wouda
 * @version 1
 */
public class WDYSEncryptedFileReader {
	/**
	 * Constructs a new file reader around the given file.
	 *
	 * @param filename The name of the encrypted file.
	 * @throws IOException When IO stuff goes wrong.
	 * @throws WDYSDecryptException When the header could not be skipped.
	 */
	public WDYSEncryptedFileReader(String filename) throws IOException, WDYSDecryptException {
		// Construct the File object of the file indicated by filename. Yeah.
		encryptedfile = new File(filename);

		// Construct a new BufferedInputStream around this baby.
		// Well... Put in a FileInputStream just to be certain ;^)
		reader = new BufferedInputStream(new FileInputStream(encryptedfile));

		// The header.
		byte[] header = new byte[12];

		// Read the first 12 bytes (the "whatdoyousee" header)...
		if(reader.read(header, 0, 12) != 12)  throw new WDYSDecryptException("Could not read header. Either a file read error, or possibly not a WDYS file, or perhaps corrupted.");

		// ...and convert it into a string.
		String wdys = new String(header, "US-ASCII");

		// Check the String whether the header is correct.
		if(!wdys.equals("whatdoyousee"))  throw new WDYSDecryptException("Header is incorrect. WDYS files are required to have a 'whatdoyousee' header. This file is not a (proper) WDYS file, or perhaps corrupted.");

		// The length attribute.
		byte[] lengthattribute = new byte[4];

		// Read the next 4 bytes (the length attribute)...
		if(reader.read(lengthattribute, 0, 4) != 4)  throw new WDYSDecryptException("Could not read the length attribute. Either a file read error, or possibly not a WDYS file, or perhaps corrupted.");

		// ...and convert it into a long (an unsigned int, as it were).
		length = HelperMethods.convertByteArrayToLong(lengthattribute);
	}


	/**
	 * Closes the file.
	 *
	 * @throws IOException When IO stuff goes wrong.
	 */
	public void close() throws IOException {
		// Yeah baby!
		reader.close();
	}

	/**
	 * Returns the length of the encrypted data chunk.
	 *
	 * @return Said length.
	 */
	public long getEncryptedDataLength() {
		// The length is the length of the file minus 16 bytes.
		return encryptedfile.length() - 16;
	}

	/**
	 * Returns the length of the decrypted data chunk (it is stored seperately). This
	 * is neccesary because the decrypted data is padded with random data to fit a
	 * multiple of 8 bytes.
	 *
	 * @return Said length.
	 */
	public long getDecryptedDataLength() {
		// The length is the length of the file minus 16 bytes.
		return length;
	}

	/**
	 * Reads and returns the next 4 bytes from the encrypted data chunk.
	 *
	 * @return What I just said above, jeez.
	 * @throws IOException When IO stuff goes wrong.
	 * @throws WDYSDecryptException When the full 4 bytes could not be read.
	 */
	public byte[] read4EncryptedDataBytes() throws IOException, WDYSDecryptException {
		// A byte array.
		byte[] quad = new byte[4];

		// Read.
		if(reader.read(quad, 0, 4) != 4) throw new WDYSDecryptException("Could not fully read next 4 bytes. Either a file read error, or possibly not a WDYS file, or perhaps corrupted.");

		// Return.
		return quad;
	}


	/**
	 * The length of the decrypted data chunk (stored seperately in the file).
	 */
	protected long length;

	/**
	 * The file.
	 */
	protected File encryptedfile;

	/**
	 * The reader. Well it's a stream actually.
	 */
	protected BufferedInputStream reader;
}