/*
	Copyright Berend "Kirk" Wouda 2004

	This file is part of WhatDoYouSee'd.

	WhatDoYouSee'd is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	WhatDoYouSee'd is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with WhatDoYouSee'd; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// The package, I mean.
package system;

// Import the IO classes.
import java.io.*;

/**
 * This class contains general methods for use in the program.
 *
 * @author Berend "Kirk" Wouda
 * @version 1
 */
public class HelperMethods {
	/**
	 * Converts a byte array standing for an unsigned int read from a file in Little
	 * Endian, into a Java long.
	 *
	 * @param unsignedintarray The byte array.
	 * @return The long representing the unsigned int.
	 * @throws IOException When the stream stuff breaks.
	 */
	public static long convertByteArrayToLong(byte[] unsignedintarray) throws IOException {
		// What follows now is a trick to invert the OS native Little Endian encoding
		// to Java's native Big Endian encoding.
		// Also, because we work with longs, some array trickage is applied :^P
		// First, we make a new byte array of 8 bytes (so that it can contain a
		// long).
		byte[] longarray = new byte[8];

		// Then we copy the passed byte array into it, at the beginning. The second
		// half of the array stays empty (all zeroes).
		System.arraycopy(unsignedintarray, 0, longarray, 0, 4);

		// Now we reverse the arrays, producing an array with a Java native long
		// encoding, only still as a byte array.
		reverseArray(longarray);

		// Now the real trick comes. We construct an InputStream over the byte array
		// and use Java's nifty DataInputStream to read the data from the array input
		// stream and convert it to a long! And we return that.
		return new DataInputStream(new ByteArrayInputStream(longarray)).readLong();
	}

	/**
	 * Converts a Java long into a byte aray standing for an unsigned int to be
	 * written to a file in Little Endian.
	 *
	 * @param javalong The long.
	 * @return The byte array representing the unsigned int in Little Endian.
	 * @throws IOException When the stream stuff breaks.
	 */
	public static byte[] convertLongToByteArray(long javalong) throws IOException {
		// What follows now is a trick to invert the Java native Big Endian encoding
		// to the OS's native Little Endian encoding.
		// Also, because we work with longs, some array trickage is applied :^P
		// First, we make new byte array of length 4 (so that it can contain an
		// unsigned int).
		byte[] unsignedintarray = new byte[4];

		// First, we make an OutputStream that outputs to a byte array.
		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		// Then, we construct a DataOutputStream over this stream, enabling us to
		// write java primitives to the byte array.
		DataOutputStream dos = new DataOutputStream(baos);

		// Write the long to the array.
		dos.writeLong(javalong);

		// Retrieve the array and copy the unsigned int part of it to the unsigned
		// int array.
		System.arraycopy(baos.toByteArray(), 4, unsignedintarray, 0, 4);

		// Reverse the byte array into Little Endian.
		reverseArray(unsignedintarray);

		// Return that.
		return unsignedintarray;
	}

	/**
	 * Reverses the given array. Note that it does not copy it, it edits the array
	 * itself.
	 *
	 * @param array The arrry to be reversed.
	 */
	public static void reverseArray(byte[] array) {
		// Temporary value holder for the swap.
		byte temp;

		// Go through half of the array. If the array length is odd, the middle value
		// is skipped, which is exactly what should happen. Yay!
		for(int index = 0; index < array.length/2; index++) {
			// Swap the value at index with its counterpart at the other end of the
			// array.
			temp = array[index];
			array[index] = array[array.length - 1 - index];
			array[array.length - 1 - index] = temp;
		}
	}
}
